@@ -16,6 +16,8 @@ LIMIT = 1000
 TEXT_COLOR = '#FFFFFF'
 DATE_FORMAT = '%Y-%m-%d'
 DATETIME_FORMAT = '%Y-%m-%dT%H:%M:%S.%fZ'
+FIRST_COLUMN_TYPES = [
+    ColumnTypes.TEXT, ColumnTypes.NUMBER, ColumnTypes.DATE, ColumnTypes.SINGLE_SELECT, ColumnTypes.AUTO_NUMBER]
 AIRTABLE_API_URL = 'https://api.airtable.com/v0/'
 ColumnTypes.BARCODE = 'barcode'
 
@@ -221,21 +223,21 @@ class ColumnsParser(object):
                     value_map[column_name].append(value)
         return value_map
 
-    def get_select_data(self, select_list):
-        return {'options': [{
-            'name': value,
+    def get_select_options(self, select_list):
+        return [{
+            'name': str(value),
             'id': self.random_num_id(),
             'color': self.random_color(),
             'textColor': TEXT_COLOR,
-        } for value in select_list]}
+        } for value in select_list]
 
     def get_column_data(self, link_map, table_name, column_name, column_type, values):
         column_data = None
         try:
             if column_type == ColumnTypes.BARCODE:
                 column_type = ColumnTypes.TEXT
-            # elif column_type == ColumnTypes.DATE and len(values[0]) == 24:
-            #     column_data = {'format': 'YYYY-MM-DD HH:mm'}
+            elif column_type == ColumnTypes.DATE:
+                column_data = {'format': 'YYYY-MM-DD'}
             elif column_type == ColumnTypes.LINK:
                 other_table_name = link_map[table_name][column_name]
                 column_data = {'other_table': other_table_name}
@@ -246,7 +248,7 @@ class ColumnsParser(object):
                         item = str(item)
                         if item not in select_list:
                             select_list.append(item)
-                column_data = self.get_select_data(select_list)
+                column_data = {'options': self.get_select_options(select_list)}
             elif column_type == ColumnTypes.COLLABORATOR:
                 if isinstance(values[0], dict):
                     column_type = ColumnTypes.SINGLE_SELECT
@@ -255,7 +257,7 @@ class ColumnsParser(object):
                         name = value['name']
                         if name not in select_list:
                             select_list.append(name)
-                    column_data = self.get_select_data(select_list)
+                    column_data = {'options': self.get_select_options(select_list)}
                 elif isinstance(values[0], list):
                     column_type = ColumnTypes.MULTIPLE_SELECT
                     select_list = []
@@ -264,9 +266,9 @@ class ColumnsParser(object):
                             name = item['name']
                             if name not in select_list:
                                 select_list.append(name)
-                    column_data = self.get_select_data(select_list)
+                    column_data = {'options': self.get_select_options(select_list)}
         except Exception as e:
-            print('[Warning] get column data error:', e)
+            print('[Warning] get', column_type.value, 'column data error:', e)
         return column_type, column_data
 
     def get_column_type(self, values):
@@ -355,29 +357,33 @@ class ColumnsParser(object):
                 if column_name == '_id':
                     continue
                 column = columns.get(column_name)
-                if not column:
+                if not column or not value:
                     continue
                 column_type = ColumnTypes(column['type'])
                 if column_type not in (ColumnTypes.SINGLE_SELECT, ColumnTypes.MULTIPLE_SELECT):
                     continue
                 if column_name not in select_value_map:
                     select_value_map[column_name] = []
-                if value is not None and value not in select_value_map[column_name]:
-                    select_value_map[column_name].append(value)
+                items = []
+                if isinstance(value, dict):
+                    items.append(value.get('name', ''))  # collaborator
+                elif isinstance(value, list):
+                    for item in value:
+                        if isinstance(item, dict):
+                            items.append(item.get('name', ''))  # collaborator
+                        else:
+                            items.append(item)
+                else:
+                    items.append(value)
+                for item in items:
+                    if item not in select_value_map[column_name]:
+                        select_value_map[column_name].append(item)
         return select_value_map
 
-    def get_select_options(self, select_list):
-        return [{
-            'name': value,
-            'id': self.random_num_id(),
-            'color': self.random_color(),
-            'textColor': TEXT_COLOR,
-        } for value in select_list]
-
     def gen_select_columns(self, select_value_map):
         select_columns = []
-        for column_name, values in select_value_map.items():
-            options = self.get_select_options(values)
+        for column_name, select_list in select_value_map.items():
+            options = self.get_select_options(select_list)
             column = {
                 'name': column_name,
                 'options': options,
@@ -429,7 +435,7 @@ class AirtableAPI(object):
 
 class AirtableConvertor(object):
 
-    def __init__(self, airtable_api_key, airtable_base_id, base, table_names, links=[]):
+    def __init__(self, airtable_api_key, airtable_base_id, base, table_names, first_columns=[], links=[]):
         """
         airtable_api_key: str
         airtable_base_id: str
@@ -440,15 +446,18 @@ class AirtableConvertor(object):
         self.airtable_api = AirtableAPI(airtable_api_key, airtable_base_id)
         self.base = base
         self.table_names = table_names
+        self.first_columns = first_columns
         self.links = links
         self.columns_parser = ColumnsParser()
         self.rows_convertor = RowsConvertor()
         self.links_convertor = LinksConvertor()
+        self.get_first_column_map()
         self.get_link_map()
 
     def convert_metadata(self):
-        self.convert_tables()
         self.get_airtable_row_map(is_demo=True)
+        self.get_airtable_column_map()
+        self.convert_tables()
         self.convert_columns()
         self.convert_rows(is_demo=True)
         self.convert_links(is_demo=True)
@@ -466,8 +475,28 @@ class AirtableConvertor(object):
         for table_name in self.table_names:
             table = self.table_map.get(table_name)
             if not table:
-                self.add_table(table_name)
-                print('[Info] Added table [ %s ]' % table_name)
+                airtable_columns = self.airtable_column_map[table_name]
+                first_column_name = self.first_column_map.get(table_name) or \
+                    airtable_columns[0]['name']
+                columns = []
+                for column in airtable_columns:
+                    if column['type'] == ColumnTypes.LINK:
+                        continue
+                    item = {
+                        'column_name': column['name'],
+                        'column_type': column['type'].value,
+                        'column_data': column['data'],
+                    }
+                    if column['name'] == first_column_name:
+                        if column['type'] not in FIRST_COLUMN_TYPES:
+                            item['column_type'] = ColumnTypes.TEXT.value
+                            item['column_data'] = None
+                        columns.insert(0, item)
+                    else:
+                        columns.append(item)
+                self.add_table(table_name, columns)
+                print(
+                    '[Info] Added table [ %s ] with %s columns' % (table_name, len(columns)))
         print('[Info] Success\n')
         time.sleep(1)
 
@@ -475,11 +504,9 @@ class AirtableConvertor(object):
         print('[Info] Convert columns')
         self.get_table_map()
         for table_name in self.table_names:
-            airtable_rows = self.airtable_row_map[table_name]
-            columns = self.columns_parser.parse(
-                self.link_map, table_name, airtable_rows)
+            airtable_columns = self.airtable_column_map[table_name]
             table = self.table_map.get(table_name)
-            for column in columns:
+            for column in airtable_columns:
                 column_name = column['name']
                 exists_column = table.get(column_name)
                 if not exists_column:
@@ -548,6 +575,14 @@ class AirtableConvertor(object):
         print('[Info] Success\n')
         time.sleep(1)
 
+    def get_first_column_map(self):
+        self.first_column_map = {}
+        for column in self.first_columns:
+            table_name = column[0]
+            column_name = column[1]
+            self.first_column_map[table_name] = column_name
+        return self.first_column_map
+
     def get_link_map(self):
         self.link_map = {}
         for link in self.links:
@@ -571,6 +606,15 @@ class AirtableConvertor(object):
         print('[Info] Success\n')
         return self.airtable_row_map
 
+    def get_airtable_column_map(self):
+        self.airtable_column_map = {}
+        for table_name in self.table_names:
+            airtable_rows = self.airtable_row_map[table_name]
+            columns = self.columns_parser.parse(
+                self.link_map, table_name, airtable_rows)
+            self.airtable_column_map[table_name] = columns
+        return self.airtable_column_map
+
     def get_table_map(self):
         self.table_map = {}
         self.column_map = {}
@@ -586,8 +630,8 @@ class AirtableConvertor(object):
         time.sleep(0.1)
         return self.table_map
 
-    def add_table(self, table_name):
-        table = self.base.add_table(table_name)
+    def add_table(self, table_name, columns):
+        table = self.base.add_table(table_name, columns=columns)
         time.sleep(0.1)
         return table
 
